package com.supwisdom.dlpay.controller

import com.supwisdom.dlpay.MvcBaseTest
import com.supwisdom.dlpay.framework.core.JwtConfig
import com.supwisdom.dlpay.framework.util.HmacUtil
import io.restassured.RestAssured
import io.restassured.RestAssured.*
import io.restassured.path.json.JsonPath.from
import org.hamcrest.Matchers.notNullValue
import org.junit.Before
import org.junit.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.boot.web.server.LocalServerPort
import org.springframework.test.context.ActiveProfiles

@ActiveProfiles("devel-pg-local")
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class SecurityControllerTest : MvcBaseTest() {
    val appid = "100001"
    val appsecret = "oUw2NmA09ficiVWD4TUQLDOkPyzQa3VzbjjsW0B2qTk="

    @LocalServerPort
    private var port: Int = 0

    @Autowired
    lateinit var jwtConfig: JwtConfig

    @Before
    fun setUp() {
        RestAssured.port = port
    }

    @Test
    fun testGetJwt() {
        val response = given().param("appid", appid)
                .`when`()
                .get("/api/auth/gettoken")
                .then()
                .body("token", notNullValue())
                .extract()

        val token = from(response.body().asString()).getString("token")
        val secret = HmacUtil.HMACSHA256(token, appsecret)

        given().param("appid", appid)
                .param("secret", secret)
                .`when`()
                .get("/api/auth/authentication")
                .then()
                .body("jwt", notNullValue())
    }

    @Test
    fun testGetJwtClient() {
        val clientid = "000030450"
        val response = given().param("appid", appid)
                .`when`()
                .get("/api/auth/gettoken/$clientid")
                .then()
                .statusCode(200)
                .body("token", notNullValue())
                .extract()

        val token = from(response.body().asString()).getString("token")
        val secret = HmacUtil.HMACSHA256(token, appsecret)

        given().param("appid", appid)
                .param("secret", secret)
                .`when`()
                .get("/api/auth/authentication/$clientid")
                .then()
                .statusCode(200)
                .body("jwt", notNullValue())
    }

    @Test
    fun testJwtRefresh() {
        val response = given().param("appid", appid)
                .`when`()
                .get("/api/auth/gettoken")
                .then()
                .statusCode(200)
                .body("token", notNullValue())
                .extract()

        val token = from(response.body().asString()).getString("token")
        val secret = HmacUtil.HMACSHA256(token, appsecret)

        given().param("appid", appid)
                .param("secret", secret)
                .`when`()
                .get("/api/auth/authentication")
                .then()
                .statusCode(200)
                .body("jwt", notNullValue())
                .extract().also {
                    val jwt = from(it.body().asString()).getString("jwt")
                    given().header(jwtConfig.header, "Bearer $jwt")
                            .`when`()
                            .get("/api/auth/refresh")
                            .then()
                            .statusCode(200)
                            .body("jwt", notNullValue())
                }
    }

}